package org.qatools.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Properties;

import org.apache.log4j.Logger;

public class SystemUtil extends QAToolsUtil {
    private static Logger logger = Logger.getLogger(SystemUtil.class);
    public static String delimiter = ",";

    /**
     * Given the full path to a file in the classpath, 
     * this loads all properties in the file into System properties and logs each key / value pair.
     * 
     * @param file
     */
    public static void loadPropertiesFileIntoSystemProperties(String filename) throws IOException {
        Properties props = new Properties();	//System.getProperties();
        props.load(ClassLoader.getSystemResourceAsStream(filename));

        logger.debug("------------------------------------------------------------------------");
        logger.debug("---------- Loading " + filename);
        logger.debug("---------- ");

        for (Entry<Object, Object> entry: props.entrySet()) {
            logger.debug("Setting '" + entry.getKey() + "' = '" + entry.getValue() + "'");
            System.setProperty((String)entry.getKey(), (String)entry.getValue());
        }
        logger.debug("------------------------------------------------------------------------");
    }

 
    // ------------------------------------------------------
    // ----- optional
    // ------------------------------------------------------
    
    /**
     * This retrieves a system property, trims any leading or trailing spaces 
     * and converts to the proper data type.
     * 
     * @param key
     * @return
     */
    public static Object getOptionalObjectProperty(String key) {
    	Object obj = System.getProperty(key);
    	logger.debug("getOptionalObjectProperty(" + key + ") = " + obj);
    	return obj;
    }
    
    /**
     * 
     * @param key
     * @return
     */
    public static String getOptionalStringProperty(String key) {
    	Object obj = getOptionalObjectProperty(key);
    	if (obj == null) {
    		return null;
    	}
    	return ((String) obj).trim();
    }

    /**
     * 
     * @param key
     * @return
     */
    public static Long getOptionalLongProperty(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return null;
    	}
    	return new Long(str);
    }

    /**
     * 
     * @param key
     * @return
     */
    public static Integer getOptionalIntegerProperty(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return null;
    	}
    	return new Integer(str);
    }
    

    /**
     * 
     * @param key
     * @return
     */
    public static Double getOptionalDoubleProperty(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return null;
    	}
    	return new Double(str);
    }
    

    /**
     * 
     * @param key
     * @return
     */
    public static Boolean getOptionalBooleanProperty(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return null;
    	}

    	Boolean bool = Boolean.FALSE;
    	if ("true".equals(str.toLowerCase().trim())) {
    		bool = Boolean.TRUE;
    	}
    	return bool;
    }

    
    // ------------------------------------------------------
    // ----- optional - delimited list of values
    // ------------------------------------------------------
    
    /**
     * 
     * @param key
     * @return
     */
    public static List<String> getOptionalStringProperties(String key) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.length()==0) {
    		return null;
    	}
    	List<String> list = ListUtil.getListFromDelimitedStr(str, delimiter);
    	if (list==null || list.size()==0) {
    		return null;
    	}
    	List<String> list2 = null;
    	for (String s : list) {
    		if (list2 == null) {
    			list2 = new ArrayList<String>();
    		}
    		list2.add(s.trim());
    	}
    	return list2;
    }

    /**
     * 
     * @param key
     * @return
     */
    public static List<Long> getOptionalLongProperties(String key) {
    	List<String> list = getOptionalStringProperties(key);
    	if (list==null || list.size()==0) {
    		return null;
    	}
    	List<Long> list2 = null;
    	for (String s : list) {
    		if (list2 == null) {
    			list2 = new LinkedList<Long>();
    		}
    		list2.add(new Long(s));
    	}
    	return list2;
    }

    /**
     * 
     * @param key
     * @return
     */
    public static List<Integer> getOptionalIntegerProperties(String key) {
    	List<String> list = getOptionalStringProperties(key);
    	if (list==null || list.size()==0) {
    		return null;
    	}
    	List<Integer> list2 = null;
    	for (String s : list) {
    		if (list2 == null) {
    			list2 = new LinkedList<Integer>();
    		}
    		list2.add(new Integer(s));
    	}
    	return list2;
    }
    

    /**
     * 
     * @param key
     * @return
     */
    public static List<Double> getOptionalDoubleProperties(String key) {
    	List<String> list = getOptionalStringProperties(key);
    	if (list==null || list.size()==0) {
    		return null;
    	}
    	List<Double> list2 = null;
    	for (String s : list) {
    		if (list2 == null) {
    			list2 = new LinkedList<Double>();
    		}
    		list2.add(new Double(s));
    	}
    	return list2;
    }
    

    /**
     * 
     * @param key
     * @return
     */
    public static List<Boolean> getOptionalBooleanProperties(String key) {
    	List<String> list = getOptionalStringProperties(key);
    	if (list==null || list.size()==0) {
    		return null;
    	}
    	List<Boolean> list2 = null;
    	for (String s : list) {
    		if (list2 == null) {
    			list2 = new LinkedList<Boolean>();
    		}
    		list2.add(new Boolean(s));
    	}
    	return list2;
    }

    
    
    // ------------------------------------------------------
    // ----- optional with default
    // ------------------------------------------------------

    /**
     * 
     * @param key
     * @return
     */
    public static String getOptionalStringProperty(String key, String defaultValue) {
    	Object obj = getOptionalObjectProperty(key);
    	if (obj == null) {
    		return defaultValue;
    	}
    	return ((String) obj).trim();
    }

    /**
     * 
     * @param key
     * @return
     */
    public static Long getOptionalLongProperty(String key, Long defaultValue) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return defaultValue;
    	}
    	return new Long(str);
    }

    /**
     * 
     * @param key
     * @return
     */
    public static Integer getOptionalIntegerProperty(String key, Integer defaultValue) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return defaultValue;
    	}
    	return new Integer(str);
    }
    

    /**
     * 
     * @param key
     * @return
     */
    public static Double getOptionalDoubleProperty(String key, Double defaultValue) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return defaultValue;
    	}
    	return new Double(str);
    }
    

    /**
     * 
     * @param key
     * @return
     */
    public static Boolean getOptionalBooleanProperty(String key, Boolean defaultValue) {
    	String str = getOptionalStringProperty(key);
    	if (str==null || str.trim().length()==0) {
    		return defaultValue;
    	}

    	Boolean bool = Boolean.FALSE;
    	if ("true".equals(str.toLowerCase().trim())) {
    		bool = Boolean.TRUE;
    	}
    	return bool;
    }
    
    
    // ------------------------------------------------------
    // ----- required
    // ------------------------------------------------------

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Object getRequiredObjectProperty(String key) throws Exception {
    	Object obj = System.getProperty(key);
    	logger.debug("getRequiredObjectProperty(" + key + ") = " + obj);
    	if (obj == null) {
    		throw new Exception("Property '" + key + "' is not defined");
    	}
    	return obj;
    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static String getRequiredStringProperty(String key) throws Exception {
    	return ((String)getRequiredObjectProperty(key)).trim();
    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Long getRequiredLongProperty(String key) throws Exception {
    	return new Long(getRequiredStringProperty(key));

    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Integer getRequiredIntegerProperty(String key) throws Exception {
    	return new Integer(getRequiredStringProperty(key));
    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Double getRequiredDoubleProperty(String key) throws Exception {
    	return new Double(getRequiredStringProperty(key));
    }

    /**
     * 
     * @param key
     * @return
     * @throws Exception
     */
    public static Boolean getRequiredBooleanProperty(String key) throws Exception {
    	String value = getRequiredStringProperty(key);
    	Boolean bool = Boolean.FALSE;
    	if ("true".equals(value.toLowerCase().trim())) {
    		bool = Boolean.TRUE;
    	}
    	return bool;
    }

}
